#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.mec_mod_interfaces.hybrid_bodies import HybridBodies
from pycatia.mec_mod_interfaces.hybrid_shape import HybridShape
from pycatia.mec_mod_interfaces.hybrid_shapes import HybridShapes
from pycatia.mec_mod_interfaces.ordered_geometrical_sets import OrderedGeometricalSets
from pycatia.mec_mod_interfaces.shapes import Shapes
from pycatia.mec_mod_interfaces.sketches import Sketches
from pycatia.system_interfaces.any_object import AnyObject


class Body(AnyObject):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     Body
                | 
                | The object that manages a sequence of shapes, a set of sketches, a set of
                | hybrid bodies, a set of ordered geometrical sets and a set of hybrid
                | shapes.
                | 
                | It belongs to the Bodies collection of a Part or HybridBody
                | object.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.body = com_object

    @property
    def hybrid_bodies(self) -> HybridBodies:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property HybridBodies() As HybridBodies (Read Only)
                | 
                |     Returns the body's HybridBodies collection.
                | 
                |     Example:
                |         The following example returns in hybridBodyColl the collection of
                |         hybrid bodies of the main body of partDoc part
                |         document:
                | 
                |          Dim body As Body
                |          Set body = partDoc.Part.Bodies.MainBody
                |          Set hybridBodyColl = body.HybridBodies

        :return: HybridBodies
        :rtype: HybridBodies
        """

        return HybridBodies(self.body.HybridBodies)

    @property
    def hybrid_shapes(self) -> HybridShapes:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property HybridShapes() As HybridShapes (Read Only)
                | 
                |     Returns the list of hybrid shapes included in the body.
                | 
                |     Returns:
                |         oHybridShapes The list of hybrid shapes in the body (@see
                |         CATIAHybridShapes
                |         for more information).
                | 
                |         Example:
                |             The following example returns in HybridShapes1 the list
                |             of
                |             hybrid shapes in the body Body1:
                | 
                |              Dim HybridShapes1 As HybridShapes
                |              Set HybridShapes1 = Body1.HybridShapes

        :return: HybridShapes
        :rtype: HybridShapes
        """

        return HybridShapes(self.body.HybridShapes)

    @property
    def in_boolean_operation(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property InBooleanOperation() As boolean (Read Only)
                | 
                |     Returns True if the body is involved in a boolean operation, else returns
                |     False.
                | 
                |     Example:
                |         The following example returns in operated True if the body body1belongs
                |         to a boolean operation.
                | 
                |          operated = body1.InBooleanOperation

        :return: bool
        :rtype: bool
        """

        return self.body.InBooleanOperation

    @property
    def ordered_geometrical_sets(self) -> OrderedGeometricalSets:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property OrderedGeometricalSets() As OrderedGeometricalSets (Read
                | Only)
                | 
                |     Returns the body's OrderedGeometricalSets collection.
                | 
                |     ometricalSetColl = Body1.OrderedGeometricalSets Example:
                |         The following example returns in OrderedGeometricalSetColl the
                |         collection of ordered geometrical set of the body Body1
                |         :
                | 
                |          Set OrderedGe

        :return: OrderedGeometricalSets
        :rtype: OrderedGeometricalSets
        """

        return OrderedGeometricalSets(self.body.OrderedGeometricalSets)

    @property
    def shapes(self) -> Shapes:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Shapes() As Shapes (Read Only)
                | 
                |     Returns the body's Shapes collection. These shapes make up the sequence of
                |     shapes that will produce an intermediate result for the part, or the final
                |     result in the case of the main body.
                | 
                |     Example:
                |         The following example returns in shapColl the collection of shapes
                |         managed by the main body of the partDoc part document:
                | 
                |          Dim body As Body
                |          Set body = partDoc.Part.Bodies.MainBody
                |          Set shapColl = body.Shapes

        :return: Shapes
        :rtype: Shapes
        """

        return Shapes(self.body.Shapes)

    @property
    def sketches(self) -> Sketches:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Sketches() As Sketches (Read Only)
                | 
                |     Returns the body's Sketches collection. These sketches are those inside the
                |     body at all levels.
                | 
                |     Example:
                |         The following example returns in skColl the collection of sketches of
                |         the main body of partDoc part document:
                | 
                |          Dim body As Body
                |          Set body = partDoc.Part.Bodies.MainBody
                |          Set skColl = body.Sketches

        :return: Sketches
        :rtype: Sketches
        """

        return Sketches(self.body.Sketches)

    def insert_hybrid_shape(self, i_hybrid_shape: HybridShape) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub InsertHybridShape(HybridShape iHybridShape)
                | 
                |     Insert a hybrid shape to the body.
                | 
                |     Parameters:
                | 
                |         iHybriShape
                |             The hybrid shape to insert. 
                | 
                |     Example:
                |         This example inserts the hybrid shape HybridShape1 to the body
                |         Body1:
                | 
                |          Body1.InsertHybridShape (HybridShape1)

        :param HybridShape i_hybrid_shape:
        :return: None
        :rtype: None
        """
        return self.body.InsertHybridShape(i_hybrid_shape.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'insert_hybrid_shape'
        # # vba_code = """
        # # Public Function insert_hybrid_shape(body)
        # #     Dim iHybridShape (2)
        # #     body.InsertHybridShape iHybridShape
        # #     insert_hybrid_shape = iHybridShape
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def __repr__(self):
        return f'Body(name="{self.name}")'



#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.

"""

from pycatia.in_interfaces.reference import Reference
from pycatia.knowledge_interfaces.dimension import Dimension
from pycatia.system_interfaces.any_object import AnyObject


class Constraint(AnyObject):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     Constraint
                |
                | A geometric constraint set for geometric elements of a part, a sketch, or a
                | product.

    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.constraint = com_object

    @property
    def angle_sector(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property AngleSector() As CatConstraintAngleSector
                |
                |     Returns or sets the constraint angle (or angular) sector. The geometric elements of an angle
                |     constraint (e.g. : 2 lines or 2 planes) divide the sketch or the space in 4 regions which are
                |     called angle or angular sectors, numbered from 0 to 3. 1 / 0 ---/--- 2/ 3 By default, the
                |     constraint is created in the sector number 0. One angle sector corresponds exactly to particular
                |     values of the Dimension.Value, the Side and the Orientation. When changing the angle sector, the
                |     Dimension.Value, Side and Orientation are also modified.
                |
                |     Parameters:
                |
                |         AngleSector=0
                |             The default sector of a constraint.
                |             Dimension.Value
                |               = angle Orientation
                |               = catCstOrientSame Side
                |               = catCstSidePositive
                |         AngleSector=1
                |             Dimension.Value
                |               = angle-180 if angle>180 abs(angle)+180 otherwise Orientation
                |               = catCstOrientOpposite Side
                |               = catCstSidePositive
                |         AngleSector=2
                |             Dimension.Value
                |               = abs(540-angle) if angle>180 180-fabs(angle) otherwise Orientation
                |               = catCstOrientOpposite Side
                |               = catCstSideNegative
                |         AngleSector=3
                |             Dimension.Value
                |               = 360-abs(angle) Orientation
                |               = catCstOrientSame Side
                |               = catCstSideNegative
                |
                |             Example:
                |                 The following example retrieves in angleSector the angle sector
                |                 of the angleCst angle constraint and then changes the angle
                |                 sector
                |
                |                  angleSector = angleCst.AngleSector
                |                  angleCst.AngleSector = 2

        :return: int
        :rtype: int
        """

        return self.constraint.AngleSector

    @angle_sector.setter
    def angle_sector(self, value: int):
        """
        :param int value:
        """

        self.constraint.AngleSector = value

    @property
    def dimension(self) -> Dimension:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property Dimension() As Dimension (Read Only)
                |
                |     Returns the constraint dimension. The dimension may be meaningless for some
                |     types of constraints such as tangency constraints, or if the constraint is not
                |     currently satisfied. Use the Status property to check whether the constraint is
                |     satisfied.
                |
                |     Example:
                |         The following example returns in cstDimension the dimension of the
                |         firstCst constraint:
                |
                |          Set cstDimension = firstCst.Dimension

        :return: Dimension
        :rtype: Dimension
        """

        return Dimension(self.constraint.Dimension)

    @property
    def distance_config(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property DistanceConfig() As CatConstraintDistConfig
                |
                |     Returns or sets the distance constraint configuration. Distance constraints
                |     between lines and cylinders offer often more degrees of freedom to geometry
                |     than acually desired. This property allows to limit these degrees of freedom
                |     without having to redefine additional constraints. This property is useless for
                |     constraints whose type is not distance.
                |
                |     Example:
                |         The following example retrieves in distCstConfig the configuration of
                |         the distCst distance constraint:
                |
                |          distCstConfig = distCst.DistanceConfig

        :return: int
        :rtype: int
        """

        return self.constraint.DistanceConfig

    @distance_config.setter
    def distance_config(self, value: int):
        """
        :param int value:
        """

        self.constraint.DistanceConfig = value

    @property
    def distance_direction(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property DistanceDirection() As CatConstraintDistDirection
                |
                |     Returns or sets the distance constraint direction. This property is useless for constraints whose
                |     type is not Distance (1). Distance constraints may be measured along a particular direction.
                |     Example:
                |         The following example retrieves in distCstDirection the configuration
                |         of the distCst distance constraint:
                | 
                |          distCstConfig = distCst.DistanceDirection

        :return: int
        :rtype: int
        """

        return self.constraint.DistanceDirection

    @distance_direction.setter
    def distance_direction(self, value: int):
        """
        :param int value:
        """

        self.constraint.DistanceDirection = value

    @property
    def mode(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property Mode() As CatConstraintMode
                |
                |     Returns or sets the constraint driving mode. For constraint types
                |     supporting the concept of value, such as distance constraints, the driving mode
                |     tells whether the constraint value actually drives the geometry position, or,
                |     conversely, is driven by it.
                |
                |     Example:
                |         The following example retrieves in currentMode the driving mode for the
                |         distCst distance constraint:
                |
                |          currentMode = distCst.Mode

        :return: int
        :rtype: int
        """

        return self.constraint.Mode

    @mode.setter
    def mode(self, value: int):
        """
        :param int value:
        """

        self.constraint.Mode = value

    @property
    def orientation(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property Orientation() As CatConstraintOrientation
                |
                |     Returns or sets the constraint orientation. This is used for constraints
                |     that involve two geometric elements and specifies the orientation for the
                |     second geometric element with regard to the first one, when several possible
                |     orientations are all satisfying the constraint.
                |
                |     Example:
                |         The following example retrieves the in distCstOrient the orientation of
                |         the distCst distance constraint:
                |
                |          distCstOrient = distCst.Orientation

        :return: int
        :rtype: int
        """

        return self.constraint.Orientation

    @orientation.setter
    def orientation(self, value: int):
        """
        :param int value:
        """

        self.constraint.Orientation = value

    @property
    def reference_axis(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property ReferenceAxis() As CatConstraintRefAxis
                |
                |     Returns or sets the constraint reference axis. AxisParallel or
                |     AxisPerpendicular constraint types define which axis they relate to through
                |     this property, which makes no sense for constraints of another
                |     type.
                |
                |     Example:
                |         The following example retrieves in refAxis the reference axis for the
                |         axisPerpCst AxisPerpendicular constraint:
                |
                |          refAxis = axisPerpCst.ReferenceAxis

        :return: int
        :rtype: int
        """

        return self.constraint.ReferenceAxis

    @reference_axis.setter
    def reference_axis(self, value: int):
        """
        :param int value:
        """

        self.constraint.ReferenceAxis = value

    @property
    def reference_type(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property ReferenceType() As CatConstraintRefType
                |
                |     Returns or sets the constraint reference type. This property is used only
                |     for Reference constraints in the Assembly context.
                |
                |     Example:
                |         The following example applies to the reference constraint refCst2 the
                |         reference type of the constraint refCst1.
                |
                |          refCst2.ReferenceType = refCst1.ReferenceType

        :return: int
        :rtype: int
        """

        return self.constraint.ReferenceType

    @reference_type.setter
    def reference_type(self, value: int):
        """
        :param int value:
        """

        self.constraint.ReferenceType = value

    @property
    def side(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property Side() As CatConstraintSide
                |
                |     Returns or sets the constraint side. Some constraint types need to
                |     relatively position the constrained geometries, when several possible
                |     configurations are all satisfying the constraint.
                |
                |     Example:
                |         The following example retrieves in distCstSide the side of the distCst
                |         distance constraint:
                |
                |          distCstSide = distCst.Side

        :return: int
        :rtype: int
        """

        return self.constraint.Side

    @side.setter
    def side(self, value: int):
        """
        :param int value:
        """

        self.constraint.Side = value

    @property
    def status(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property Status() As CatConstraintStatus (Read Only)
                |
                |     Returns the constraint status. The constraint status is a diagnosis on
                |     whether the constraint is satisfied.
                |
                |     Example:
                |         The following example retrieves the status of the distCst distance
                |         constraint.
                |
                |          distCstSts = distCst.Status

        :return: int
        :rtype: int
        """

        return self.constraint.Status

    @property
    def type(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property Type() As CatConstraintType (Read Only)
                |
                |     Returns the constraint type.
                |
                |     Example:
                |         The following example returns in cstType the type of the firstCst
                |         constraint:
                |
                |          cstType = firstCst.Type

        :return: int
        :rtype: int
        """

        return self.constraint.Type

    def activate(self) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Sub Activate()
                |
                |     Unsuppresses a constraint for the update process. An activated constraint
                |     is again taken into account for the calculation of the part or product.
                |
                | Example:
                |     The following example es the pad1 pad:
                | Example:
                |     The following example activates the tangencyCst constraint
                |     :
                |
                |      tangencyCst.Activate

        :return: None
        :rtype: None
        """
        return self.constraint.Activate()

    def deactivate(self) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Sub Deactivate()
                |
                |     Suppresses the constraint from being updated. A deactivated constraint is
                |     not taken into account for the calculation of the part or of the product.
                |
                |
                | Example:
                |     The following example deactivates the tangencyCst constraint from being
                |     updated:
                |
                |      tangencyCst.Deactivate

        :return: None
        :rtype: None
        """
        return self.constraint.Deactivate()

    def get_constraint_element(self, i_element_number: int) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func GetConstraintElement(long iElementNumber) As
                | Reference
                |
                |     Reads an element of a constraint.
                |
                |     Parameters:
                |
                |         iElementNumber
                |             The number of the element of the constraint to be read. (1 for the first element,2 for
                |             the second, 3 for the third). Notice it must not exceed the total number of elements of
                |             the constraint. (eg : not allowed to read the third element of a tangency).
                |         oCurrentElement
                |             An element of the constraint.
                |
                |     Example:
                |         The following example reads the first element of a
                |         constraint
                |
                |          Dim reference1 As Reference
                |          reference1=tangencyCst.GetConstraintElement( 1 )

        :param int i_element_number:
        :return: Reference
        :rtype: Reference
        """
        return Reference(self.constraint.GetConstraintElement(i_element_number))

    def get_constraint_visu_location(self, o_anchor_point: tuple, o_anchor_vector: tuple) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Sub GetConstraintVisuLocation(CATSafeArrayVariant
                | oAnchorPoint,
                | CATSafeArrayVariant oAnchorVector)
                |
                |     Returns the constraint visualisation location. When displayed on screen,
                |     the constraint is visualized as a dimension positioned close to the constrained
                |     geometric element(s). This method retrieves the data used to position this
                |     representation within the 3D space.
                |
                |     Parameters:
                |
                |         oAnchorPoint
                |             A Safe Array made up of 3 doubles: X, Y, Z, representing the
                |             coordinates in model space of the point where the constraint value is
                |             displayed.
                |         oAnchorVector
                |             A Safe Array made up of 3 doubles : X, Y, Z, representing the vector normal to the plane
                |             onto which the constraint value is displayed.
                |
                |     Example:
                |         The following example retrieves in anchorPt the anchor point of the
                |         tangencyCst tangency constraint:
                |
                |          Dim anchorPoint(2)
                |          Dim anchorVector(2)
                |          tangencyCst.ConstraintVisuLocation
                |          anchorPoint,vectorPoint

        :param tuple o_anchor_point:
        :param tuple o_anchor_vector:
        :return: None
        :rtype: None
        """
        return self.constraint.GetConstraintVisuLocation(o_anchor_point, o_anchor_vector)
        # # # # Autogenerated comment:
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'get_constraint_visu_location'
        # # vba_code = """
        # # Public Function get_constraint_visu_location(constraint)
        # #     Dim oAnchorPoint (2)
        # #     constraint.GetConstraintVisuLocation oAnchorPoint
        # #     get_constraint_visu_location = oAnchorPoint
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def is_inactive(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func IsInactive() As boolean
                |
                |     Indicates whether a constraint is suppressed from the update process. A
                |     suppressed constraint is not taken into account for the calculation of part or
                |     the product. The method returns True if the constraint is not active, False if
                |     the constraint is active.
                | Example:
                |     The following example returns in isInactive whether the tangencyCst
                |     constraint is suppressed from the update process:
                |
                |      Set isInactive = tangencyCst.IsInactive

        :return: bool
        :rtype: bool
        """
        return self.constraint.IsInactive()

    def set_constraint_element(self, i_element_number: int, i_new_element: Reference) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Sub SetConstraintElement(long iElementNumber,
                | Reference iNewElement)
                |
                |     Replaces an element of a constraint.
                |
                |     Parameters:
                |
                |         iElementNumber
                |             The number of the element of the constraint to replace. (1 for the
                |             first element,2 for the second, 3 for the third).
                |         iNewElement
                |             A new element of the constraint.
                |
                |     Example:
                |         The following example changes the second element of a
                |         constraint
                |
                |          Dim reference1 As Reference
                |          tangencyCst.SetConstraintElement ( 2, reference1)

        :param int i_element_number:
        :param Reference i_new_element:
        :return: None
        :rtype: None
        """
        return self.constraint.SetConstraintElement(i_element_number, i_new_element.com_object)
        # # # # Autogenerated comment:
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_constraint_element'
        # # vba_code = """
        # # Public Function set_constraint_element(constraint)
        # #     Dim iElementNumber (2)
        # #     constraint.SetConstraintElement iElementNumber
        # #     set_constraint_element = iElementNumber
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def set_constraint_visu_location(self, i_new_x: float, i_new_y: float, i_new_z: float) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Sub SetConstraintVisuLocation(double iNewX,
                | double iNewY,
                | double iNewZ)
                |
                |     Sets a new location for the constraint visualization.
                |
                |     Parameters:
                |
                |         iNewX
                |             The new value for the constraint anchor point X coordinate
                |
                |         iNewY
                |             The new value for the constraint anchor point Y coordinate
                |
                |         iNewZ
                |             The new value for the constraint anchor point Z coordinate
                |
                |
                |     Example:
                |         The following example changes the anchor point coordinates to
                |         10,0,0
                |
                |          tangencyCst.SetConstraintVisuLocation 10,0,0

        :param float i_new_x:
        :param float i_new_y:
        :param float i_new_z:
        :return: None
        :rtype: None
        """
        return self.constraint.SetConstraintVisuLocation(i_new_x, i_new_y, i_new_z)

    def __repr__(self):
        return f'Constraint(name="{self.name}")'



#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-09-25 14:34:21.593357

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""
from pycatia.funct_system_interfaces.functional_element import FunctionalElement


class FunctionalPosition(FunctionalElement):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     CATFunctSystemItf.FunctionalElement
                |                         FunctionalPosition
                | 
                | Represents a Functional Position.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.functional_position = com_object

    @property
    def x(self) -> float:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property X() As double (Read Only)
                | 
                |     Returns the X coordinate.

        :return: float
        :rtype: float
        """

        return self.functional_position.X

    @property
    def y(self) -> float:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property Y() As double (Read Only)
                | 
                |     Returns the Y coordinate.

        :return: float
        :rtype: float
        """

        return self.functional_position.Y

    def set_coords(self, i_x: float, i_y: float) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub SetCoords(double iX,
                | double iY)
                | 
                |     Sets the coordinates.
                | 
                |     Parameters:
                | 
                |         iX
                |             the x coordinate. 
                |         iY
                |             the y coordinate

        :param float i_x:
        :param float i_y:
        :return: None
        :rtype: None
        """
        return self.functional_position.SetCoords(i_x, i_y)

    def __repr__(self):
        return f'FunctionalPosition(name="{self.name}")'



#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-07-06 14:02:20.222384

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.hybrid_shape_interfaces.plane import Plane
from pycatia.in_interfaces.reference import Reference
from pycatia.knowledge_interfaces.length import Length


class HybridShapePlaneOffset(Plane):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     MecModInterfaces.HybridShape
                |                         CATGSMIDLItf.Plane
                |                             HybridShapePlaneOffset
                | 
                | Offset plane.
                | Role: Allows to access data of the plane feature created with an offset to
                | another plane.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.hybrid_shape_plane_offset = com_object

    @property
    def offset(self) -> Length:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Offset() As Length (Read Only)
                | 
                |     Returns the offset value.

        :return: Length
        :rtype: Length
        """

        return Length(self.hybrid_shape_plane_offset.Offset)

    @property
    def orientation(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Orientation() As long
                | 
                |     Returns or sets the plane orientation.
                |     Role: The orientation allows you to invert the plane from the reference
                |     plane.
                |     Legal values: the orientation is 1 if the plane orientation is not
                |     inverted, and -1 otherwise.

        :return: int
        :rtype: int
        """

        return self.hybrid_shape_plane_offset.Orientation

    @orientation.setter
    def orientation(self, value: int):
        """
        :param int value:
        """

        self.hybrid_shape_plane_offset.Orientation = value

    @property
    def ref_plane(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Plane() As Reference
                | 
                |     Returns or sets the reference plane.
                |     Sub-element(s) supported (see Boundary object): PlanarFace.

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.hybrid_shape_plane_offset.Plane)

    @ref_plane.setter
    def ref_plane(self, reference_plane: Reference):
        """
        :param Reference reference_plane:
        """

        self.hybrid_shape_plane_offset.Plane = reference_plane.com_object

    def __repr__(self):
        return f'HybridShapePlaneOffset(name="{self.name}")'



#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-07-06 14:02:20.222384

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.hybrid_shape_interfaces.line import Line
from pycatia.in_interfaces.reference import Reference
from pycatia.knowledge_interfaces.length import Length


class HybridShapeLinePtPt(Line):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     MecModInterfaces.HybridShape
                |                         CATGSMIDLItf.Line
                |                             HybridShapeLinePtPt
                | 
                | Represents the hybrid shape line between two points feature
                | object.
                | Role: To access the data of the line feature created between two
                | points.
                | 
                | Use the CATIAHybridShapeFactory to create a HybridShapeLinePtPt
                | object.
                | 
                | See also:
                |     Reference 
                | See also:
                |     Length 
                | See also:
                |     HybridShapeFactory
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.hybrid_shape_line_pt_pt = com_object

    @property
    def begin_offset(self) -> Length:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property BeginOffset() As Length (Read Only)
                | 
                |     Returns the start length of the line.
                |     Start length : extension of the line, beginning at the starting point
                | 
                |     Example:
                |         This example retrieves in oStart the beginning offset length for the
                |         LinePtPt hybrid shape feature.
                | 
                |          Dim oStart As  CATIALength 
                |          Set oStart = LinePtPt.BeginOffset

        :return: Length
        :rtype: Length
        """

        return Length(self.hybrid_shape_line_pt_pt.BeginOffset)

    @property
    def end_offset(self) -> Length:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property EndOffset() As Length (Read Only)
                | 
                |     Returns the end length of the line.
                |     End length : extension of the line, beginning at the ending point
                | 
                |     Example:
                |         This example retrieves in oEnd the starting length for the LinePtPt
                |         hybrid shape feature.
                | 
                |          Dim oEnd As  CATIALength 
                |          Set oEnd = LinePtPt.EndOffset

        :return: Length
        :rtype: Length
        """

        return Length(self.hybrid_shape_line_pt_pt.EndOffset)

    @property
    def pt_extremity(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property PtExtremity() As Reference
                | 
                |     Returns or Sets the extremity point of the LinePtPt(Second
                |     Point).
                |     Sub-element(s) supported (see Boundary object): Vertex.
                | 
                |     Example:
                |         This example retrieves in oPtExtremity the ending point for the
                |         LinePtPt hybrid shape feature.
                | 
                |          Dim oPtExtremity As Reference 
                |          Set oPtExtremity = LinePtPt.PtExtremity

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.hybrid_shape_line_pt_pt.PtExtremity)

    @pt_extremity.setter
    def pt_extremity(self, pt_reference: Reference):
        """
        :param Reference pt_reference:
        """

        self.hybrid_shape_line_pt_pt.PtExtremity = pt_reference.com_object

    @property
    def pt_origin(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property PtOrigine() As Reference
                | 
                |     Returns or Sets the origin point of the LinePtPt(First
                |     point).
                |     Sub-element(s) supported (see Boundary object): Vertex.
                | 
                |     Example:
                |         This example retrieves in oPtOrigine the initial point for the LinePtPt
                |         hybrid shape feature.
                | 
                |          Dim oPtOrigine As Reference 
                |          Set oPtOrigine = LinePtPt.PtOrigine

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.hybrid_shape_line_pt_pt.PtOrigine)

    @pt_origin.setter
    def pt_origin(self, pt_reference: Reference):
        """
        :param Reference pt_reference:
        """

        self.hybrid_shape_line_pt_pt.PtOrigine = pt_reference.com_object

    @property
    def support(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Support() As Reference
                | 
                |     Returns or Sets the supporting surface.
                |     Note: the support surface is not mandatory for LinePtPt
                | 
                |     Sub-element(s) supported (see Boundary object): Face.
                | 
                |     Example:
                |         This example retrieves in oSurface the supporting surface (if it exist)
                |         for the LinePtPt hybrid shape feature.
                | 
                |          Dim oSurface As Reference 
                |          Set oSurface = LinePtPt.Surface

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.hybrid_shape_line_pt_pt.Support)

    @support.setter
    def support(self, support_reference: Reference):
        """
        :param Reference support_reference:
        """

        self.hybrid_shape_line_pt_pt.Support = support_reference.com_object

    def get_length_type(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Func GetLengthType() As long
                | 
                |     Gets the length type Default is 0.
                | 
                |     Parameters:
                | 
                |         oType
                |             The length type = 0 : length - the line is limited by its extremities =
                |             1 : infinite - the line is infinite = 2 : infinite start point - the line is infinite on
                |             the side of the start point = 3 : infinite end point - the line is infinite on the side of
                |             the end point

        :return: int
        :rtype: int
        """
        return self.hybrid_shape_line_pt_pt.GetLengthType()

    def get_symmetrical_extension(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Func GetSymmetricalExtension() As boolean
                | 
                |     Gets whether the symmetrical extension of the line is
                |     active.
                | 
                |     Parameters:
                | 
                |         oSym
                |             Symetry flag

        :return: bool
        :rtype: bool
        """
        return self.hybrid_shape_line_pt_pt.GetSymmetricalExtension()

    def remove_support(self) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub RemoveSupport()
                | 
                |     Removes the support surface.

        :return: None
        :rtype: None
        """
        return self.hybrid_shape_line_pt_pt.RemoveSupport()

    def set_length_type(self, i_type: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub SetLengthType(long iType)
                | 
                |     Sets the length type Default is 0.
                | 
                |     Parameters:
                | 
                |         iType
                |             The length type = 0 : length - the line is limited by its extremities =
                |             1 : infinite - the line is infinite = 2 : infinite start point - the line is infinite on
                |             the side of the start point = 3 : infinite end point - the line is infinite on the side
                |             of the end point

        :param int i_type:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_line_pt_pt.SetLengthType(i_type)

    def set_symmetrical_extension(self, i_sym: bool) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub SetSymmetricalExtension(boolean iSym)
                | 
                |     Sets the symmetrical extension of the line (start = -end).
                | 
                |     Parameters:
                | 
                |         iSym
                |             Symetry flag

        :param bool i_sym:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_line_pt_pt.SetSymmetricalExtension(i_sym)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_symmetrical_extension'
        # # vba_code = """
        # # Public Function set_symmetrical_extension(hybrid_shape_line_pt_pt)
        # #     Dim iSym (2)
        # #     hybrid_shape_line_pt_pt.SetSymmetricalExtension iSym
        # #     set_symmetrical_extension = iSym
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def __repr__(self):
        return f'HybridShapeLinePtPt(name="{self.name}")'



#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-07-06 14:02:20.222384

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.in_interfaces.reference import Reference
from pycatia.mec_mod_interfaces.hybrid_shape import HybridShape


class HybridShapeTransfer(HybridShape):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     MecModInterfaces.HybridShape
                |                         HybridShapeTransfer
                | 
                | Represents the hybrid shape Transfer feature object.
                | Role: To access the data of the hybrid shape Transfer feature object. This data
                | includes:
                | 
                |     The element to transfer
                |     The surface to unfold
                |     The unfolded surface
                | 
                | Use the CATIAHybridShapeFactory to create a HybridShapeTransfer
                | object.
                | 
                | See also:
                |     HybridShapeFactory
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.hybrid_shape_transfer = com_object

    @property
    def element_to_transfer(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property ElementToTransfer() As Reference
                | 
                |     Returns or sets the element to transfer.

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.hybrid_shape_transfer.ElementToTransfer)

    @element_to_transfer.setter
    def element_to_transfer(self, reference_element: Reference):
        """
        :param Reference reference_element:
        """

        self.hybrid_shape_transfer.ElementToTransfer = reference_element.com_object

    @property
    def surface_to_unfold(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property SurfaceToUnfold() As Reference
                | 
                |     Returns or sets the surface to unfold.

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.hybrid_shape_transfer.SurfaceToUnfold)

    @surface_to_unfold.setter
    def surface_to_unfold(self, reference_surface: Reference):
        """
        :param Reference reference_surface:
        """

        self.hybrid_shape_transfer.SurfaceToUnfold = reference_surface.com_object

    @property
    def type_of_transfer(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property TypeOfTransfer() As long
                | 
                |     Returns or sets the type of transfer.
                | 
                |         0= The type of surface is not defined
                |         1= The type of transfer is folded to unfolded
                |         2= The type of surface is unfolded to folded

        :return: int
        :rtype: int
        """

        return self.hybrid_shape_transfer.TypeOfTransfer

    @type_of_transfer.setter
    def type_of_transfer(self, value: int):
        """
        :param int value:
        """

        self.hybrid_shape_transfer.TypeOfTransfer = value

    @property
    def unfold_type(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property UnfoldType() As long
                | 
                |     Returns or sets the type of unfold to take into account during
                |     transfer.
                | 
                |         0= The type is undefined
                |         1= The surface to unfold is ruled,
                |         2= the surface to unfold is all

        :return: int
        :rtype: int
        """

        return self.hybrid_shape_transfer.UnfoldType

    @unfold_type.setter
    def unfold_type(self, value: int):
        """
        :param int value:
        """

        self.hybrid_shape_transfer.UnfoldType = value

    @property
    def unfolded_surface(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property UnfoldedSurface() As Reference
                | 
                |     Returns or sets the unfolded surface.

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.hybrid_shape_transfer.UnfoldedSurface)

    @unfolded_surface.setter
    def unfolded_surface(self, reference_surface: Reference):
        """
        :param Reference reference_surface:
        """

        self.hybrid_shape_transfer.UnfoldedSurface = reference_surface.com_object

    def __repr__(self):
        return f'HybridShapeTransfer(name="{self.name}")'



#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.in_interfaces.viewer_2d import Viewer2D


class SpecsViewer(Viewer2D):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     InfInterfaces.Viewer
                |                         InfInterfaces.Viewer2D
                |                             SpecsViewer
                | 
                | Represents the specification tree viewer.
                | This viewer displays the document's specification tree according to the chosen
                | layout, and can only be included in a SpecsAndGeomWindow
                | object.
                | 
                | See also:
                |     CatSpecsLayout, SpecsAndGeomWindow
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.specs_viewer = com_object

    @property
    def layout(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Layout() As CatSpecsLayout
                | 
                |     Returns or sets the specification tree layout.
                | 
                |     Example:
                |         This example sets the specification tree layout for the SpecsTreeViewer
                |         specification tree viewer to
                |         catSpecsViewerHorizontalCentered.
                | 
                |          SpecsTreeViewer.Layout = catSpecsViewerHorizontalCentered

        :return: int
        :rtype: int
        """

        return self.specs_viewer.Layout

    @layout.setter
    def layout(self, value: int):
        """
        :param int value:
        """

        self.specs_viewer.Layout = value

    def __repr__(self):
        return f'SpecsViewer(name="{ self.name }")'



#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-07-06 14:02:20.222384

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.system_interfaces.any_object import AnyObject


class Move(AnyObject):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     Move
                | 
                | Represents an object associated with any kind of objects which retrieves the
                | corresponding movable object.
                | An object can be moved only if it is independent from other objects. For
                | example, a pad can be moved, and is as such a movable object, while an edge
                | cannot be moved individually. It should be moved with all the objects making up
                | the pad to which it belongs as a whole. In this case, the underlying edge's
                | movable object is the pad.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.move = com_object

    @property
    def movable_object(self) -> 'Move':
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic help

                | MovableObject
                | o Property MovableObject() As   (Read Only)
                | 
                | Returns the movable object associated with the used object.
                | Example: This example retrieves the myMovableObject from
                | Move object associated with MyObject. Dim myMovableObject As
                | Move Set myMovableObject = MyObject.Move.MovableObject
                |

        :return: Move
        :rtype: Move
        """

        return Move(self.move.MovableObject)

    def apply(self, i_transformation_array: tuple) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic help

                | Apply
                | o Sub Apply(iTransformationArray)
                | 
                | Applies a move transformation to a movable object.
                |
                | Parameters:
                | TransformationArray
                |    The linear transformation array initialized successively by the
                |    four columns of the transformation matrix.
                |    The first nine components represent the rotation matrix.
                |    The last three components represent the translation vector.
                |
                | Examples:
                | This example applies the transformation (45 degrees-rotation
                | around the x axis and a translation) stored in
                | TransformationArray to the Move object associated with
                | myMovableObject:
                | Dim TransformationArray( 11 )
                | 'Rotation( 45 degrees around the x axis) components
                | TransformationArray( 0 ) = 1.000
                | TransformationArray( 1 ) = 0
                | TransformationArray( 2 ) = 0
                | TransformationArray( 3 ) = 0
                | TransformationArray( 4 ) = 0.707
                | TransformationArray( 5 ) = 0.707
                | TransformationArray( 6 ) = 0
                | TransformationArray( 7 ) = -0.707
                | TransformationArray( 8 ) = 0.707
                | 'Translation Vector (10,20,30)
                | TransformationArray( 9 ) = 10.000
                | TransformationArray( 10 ) = 20.000
                | TransformationArray( 11 ) = 30.000
                | MyMovableObject.Move.Apply TransformationArray

        :param tuple i_transformation_array:
        :return: None
        :rtype: None
        """
        return self.move.Apply(i_transformation_array)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'apply'
        # # vba_code = """
        # # Public Function apply(move)
        # #     Dim iTransformationArray (2)
        # #     move.Apply iTransformationArray
        # #     apply = iTransformationArray
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def __repr__(self):
        return f'Move(name="{self.name}")'



#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-07-06 14:02:20.222384

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.in_interfaces.reference import Reference
from pycatia.knowledge_interfaces.angle import Angle
from pycatia.mec_mod_interfaces.hybrid_shape import HybridShape


class HybridShapeRevol(HybridShape):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     MecModInterfaces.HybridShape
                |                         HybridShapeRevol
                | 
                | The Revol feature : an Revol is made up of a face to process and one Revol parameter.
                | Role: To access the data of the hybrid shape revol feature
                | object.
                | 
                | LICENSING INFORMATION: Creation of volume result requires GSO
                | License
                | if GSO License is not granted , setting of Volume context has not
                | effect
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.hybrid_shape_revol = com_object

    @property
    def axis(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Axis() As Reference
                | 
                |     Role: To get_Axis on the object.
                | 
                |     Parameters:
                | 
                |         oDir
                |             return value for CATScript applications, with (IDLRETVAL) function
                |             type 
                | 
                |     See also:
                |         Reference 
                |     Returns:
                |         HRESULT S_OK if Ok E_FAIL else return error code for C++
                |         Implementations 
                |     See also:
                |         HybridShapeFactory

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.hybrid_shape_revol.Axis)

    @axis.setter
    def axis(self, reference_axis: Reference):
        """
        :param Reference reference_axis:
        """

        self.hybrid_shape_revol.Axis = reference_axis.com_object

    @property
    def begin_angle(self) -> Angle:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property BeginAngle() As Angle (Read Only)
                | 
                |     Role: To get_BeginAngle on the object.
                | 
                |     Parameters:
                | 
                |         oAngle
                |             return value for CATScript applications, with (IDLRETVAL) function
                |             type 
                | 
                |     See also:
                |         Angle 
                |     Returns:
                |         HRESULT S_OK if Ok E_FAIL else return error code for C++
                |         Implementations 
                |     See also:
                |         HybridShapeFactory

        :return: Angle
        :rtype: Angle
        """

        return Angle(self.hybrid_shape_revol.BeginAngle)

    @property
    def context(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Context() As long
                | 
                |     Returns or sets the context on Revolve feature.
                |     Legal values:
                | 
                |         0 This option creates surface of revolution.
                |         1 This option creates volume of revolution.
                | 
                | 
                |     Note: Setting volume result requires GSO License.
                | 
                |     Example:
                |         This example retrieves in oContext the context for the Revol hybrid
                |         shape feature.
                | 
                |          Dim oContext
                |          Set oContext = Revol.Context

        :return: int
        :rtype: int
        """

        return self.hybrid_shape_revol.Context

    @context.setter
    def context(self, value: int):
        """
        :param int value:
        """

        self.hybrid_shape_revol.Context = value

    @property
    def end_angle(self) -> Angle:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property EndAngle() As Angle (Read Only)
                | 
                |     Role: To get_EndAngle on the object.
                | 
                |     Parameters:
                | 
                |         oAngle
                |             return value for CATScript applications, with (IDLRETVAL) function
                |             type 
                | 
                |     See also:
                |         Angle 
                |     Returns:
                |         HRESULT S_OK if Ok E_FAIL else return error code for C++
                |         Implementations 
                |     See also:
                |         HybridShapeFactory

        :return: Angle
        :rtype: Angle
        """

        return Angle(self.hybrid_shape_revol.EndAngle)

    @property
    def first_limit_type(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property FirstLimitType() As long
                | 
                |     Returns or sets the First limit type.
                |     Legal values:
                | 
                |     0
                |         Unknown Limit type.
                |     1
                |         Limit type is Dimension. It implies that limit is defined by
                |         length
                |     2
                |         Limit type is UptoElement. It implies that limit is defined by a
                |         geometrical element
                | 
                | Example:
                |     This example retrieves in oLim1Type the first limit type for the Revolve
                |     hybrid shape feature.
                | 
                |      Dim oLim1Type
                |      Set oLim1Type = Revolve.FirstLimitType

        :return: int
        :rtype: int
        """

        return self.hybrid_shape_revol.FirstLimitType

    @first_limit_type.setter
    def first_limit_type(self, value: int):
        """
        :param int value:
        """

        self.hybrid_shape_revol.FirstLimitType = value

    @property
    def first_upto_element(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property FirstUptoElement() As Reference
                | 
                |     Returns or sets the First up-to element used to limit
                |     Revolution.
                | 
                |     Example:
                |         This example retrieves in Lim1Elem the First up-to element for the
                |         Revolve hybrid shape feature.
                | 
                |          Dim Lim1Elem As Reference 
                |          Set Lim1Elem = Revolve.FirstUptoElement

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.hybrid_shape_revol.FirstUptoElement)

    @first_upto_element.setter
    def first_upto_element(self, reference_element: Reference):
        """
        :param Reference reference_element:
        """

        self.hybrid_shape_revol.FirstUptoElement = reference_element.com_object

    @property
    def orientation(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Orientation(boolean iOrientation)
                | 
                |     Gets or sets orientation of the revolution.
                |     Orientation
                |     TRUE : The natural orientation of the axis is taken.
                |     FALSE : The opposite orientation is taken This example retrieves in IsInverted orientation of the
                |     revolution for the Revol hybrid shape feature.
                | 
                |      Dim IsInverted As boolean
                |      IsInverted = Revol.Orientation

        :return: bool
        :rtype: bool
        """

        return self.hybrid_shape_revol.Orientation

    @orientation.setter
    def orientation(self, value: bool):
        """
        :param bool value:
        """

        self.hybrid_shape_revol.Orientation = value

    @property
    def profile(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Profil() As Reference
                | 
                |     Role: To get_Profil on the object.
                | 
                |     Parameters:
                | 
                |         oProfil
                |             return value for CATScript applications, with (IDLRETVAL) function
                |             type 
                | 
                |     See also:
                |         Reference 
                |     Returns:
                |         HRESULT S_OK if Ok E_FAIL else return error code for C++
                |         Implementations 
                |     See also:
                |         HybridShapeFactory

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.hybrid_shape_revol.Profil)

    @profile.setter
    def profile(self, reference_profile: Reference):
        """
        :param Reference reference_profile:
        """

        self.hybrid_shape_revol.Profil = reference_profile.com_object

    @property
    def second_limit_type(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property SecondLimitType() As long
                | 
                |     Returns or sets the Second limit type.
                |     Legal values:
                | 
                |     0
                |         Unknown Limit type.
                |     1
                |         Limit type is Dimension. It implies that limit is defined by
                |         length
                |     2
                |         Limit type is UptoElement. It implies that limit is defined by a
                |         geometrical element
                | 
                | Example:
                |     This example retrieves in oLim2Type the second limit type for the Revolve
                |     hybrid shape feature.
                | 
                |      Dim oLim2Type
                |      Set oLim2Type = RevolveSecondLimitType

        :return: int
        :rtype: int
        """

        return self.hybrid_shape_revol.SecondLimitType

    @second_limit_type.setter
    def second_limit_type(self, value: int):
        """
        :param int value:
        """

        self.hybrid_shape_revol.SecondLimitType = value

    @property
    def second_upto_element(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property SecondUptoElement() As Reference
                | 
                |     Returns or sets the Second up-to element used to limit
                |     Revolution.
                | 
                |     Example:
                |         This example retrieves in Lim2Elem the Second up-to element for the
                |         Revolve hybrid shape feature.
                | 
                |          Dim Lim2Elem As Reference 
                |          Set Lim2Elem = Revolve.SecondUptoElement

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.hybrid_shape_revol.SecondUptoElement)

    @second_upto_element.setter
    def second_upto_element(self, reference_element: Reference):
        """
        :param Reference reference_element:
        """

        self.hybrid_shape_revol.SecondUptoElement = reference_element.com_object

    def __repr__(self):
        return f'HybridShapeRevol(name="{self.name}")'



#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""
from typing import Iterator

from pycatia.navigator_interfaces.group import Group
from pycatia.system_interfaces.collection import Collection
from pycatia.types.general import cat_variant


class Groups(Collection):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.Collection
                |                     Groups
                | 
                | A collection of all groups currently managed by the
                | application.
                | 
                | The method Product.GetTechnologicalObject ("Groups") on the root product
                | retrieves this collection.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object, child_object=Group)
        self.groups = com_object

    def add(self) -> Group:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Func Add() As Group
                | 
                |     Creates an empty Group and adds it to the Groups
                |     Collection.
                | 
                |     Returns:
                |         The created group 
                |     Example:
                | 
                |              This example creates a new group in the TheGroups
                |              collection.
                |             
                | 
                |             Dim NewGroup As Group
                |             Set NewGroup = TheGroups.Add

        :return: Group
        :rtype: Group
        """
        return Group(self.groups.Add())

    def add_from_sel(self) -> Group:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Func AddFromSel() As Group
                | 
                |     Creates a Group containing all products in the selection and adds it to the
                |     Groups Collection.
                | 
                |     Returns:
                |         The created group 
                |     Example:
                | 
                |              This example creates a new group containing all products in the
                |              selection
                |             in the TheGroups collection.
                |             
                | 
                |             Dim NewGroup As Group
                |             Set NewGroup = TheGroups.AddFromSel

        :return: Group
        :rtype: Group
        """
        return Group(self.groups.AddFromSel())

    def all_leaves(self) -> Group:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Func AllLeaves() As Group
                | 
                |     Returns a group which contains all the terminal nodes of the current root
                |     product.
                | 
                |     Example:
                | 
                |              This example retrieves the group in the TheGroups
                |              collection.
                |             
                | 
                |             Dim AllLeavesGroup As Group
                |             Set AllLeavesGroup = TheGroups.AllLeaves

        :return: Group
        :rtype: Group
        """
        return Group(self.groups.AllLeaves())

    def item(self, i_index: cat_variant) -> Group:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Func Item(CATVariant iIndex) As Group
                | 
                |     Returns a group using its index or its name from the Groups
                |     collection.
                | 
                |     Parameters:
                | 
                |         iIndex
                |             The index or the name of the Group to retrieve from the collection
                |             of groups. As a numerics, this index is the rank of the Group in the
                |             collection. The index of the first Group in the collection is 1, and the index
                |             of the last Group is Count. As a string, it is the name you assigned to the
                |             Group. 
                | 
                |     Returns:
                |         The retrieved Group 
                |     Example:
                | 
                |              This example retrieves in ThisGroup the ninth
                |              Group,
                |             and in ThatGroup the Group named
                |             Group3 from the TheGroups collection. 
                |             
                | 
                |             Dim ThisGroup As Group
                |             Set ThisGroup = TheGroups.Item(9)
                |             Dim ThatGroup As Group
                |             Set ThatGroup = TheGroups.Item("Group3")

        :param cat_variant i_index:
        :return: Group
        :rtype: Group
        """
        return Group(self.groups.Item(i_index))

    def remove(self, i_index: cat_variant) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub Remove(CATVariant iIndex)
                | 
                |     Removes a group from the Groups collection.
                | 
                |     Parameters:
                | 
                |         iIndex
                |             The index or the name of the Group to retrieve from he collection
                |             of groups. As a numerics, this index is the rank of the Group in the
                |             collection. The index of the first Group in the collection is 1, and the index
                |             of the last Group is Count. As a string, it is the name you assigned to the
                |             Group. 
                | 
                |     Example:
                | 
                |              The following example removes the tenth Group and the Group
                |              named
                |             Group2 from the TheGroups collection.
                |             
                | 
                |             TheGroups.Remove(10)
                |             TheGroups.Remove("Group2")

        :param cat_variant i_index:
        :return: None
        :rtype: None
        """
        return self.groups.Remove(i_index)

    def __getitem__(self, n: int) -> Group:
        if (n + 1) > self.count:
            raise StopIteration

        return Group(self.groups.item(n + 1))

    def __iter__(self) -> Iterator[Group]:
        for i in range(self.count):
            yield self.child_object(self.com_object.item(i + 1))

    def __repr__(self):
        return f'Groups(name="{self.name}")'
